@@ -32,7 +32,9 @@ activate_entry (GtkWidget * widget, gpointer user_data)
 {
 	g_return_if_fail(INDICATOR_IS_OBJECT(user_data));
 	gpointer entry = g_object_get_data(G_OBJECT(widget), ENTRY_DATA_NAME);
-	g_return_if_fail(entry == NULL);
+	if (entry == NULL) {
+		g_debug("Activation on: (null)");
+	}
 
 	indicator_object_entry_activate(INDICATOR_OBJECT(user_data), (IndicatorObjectEntry *)entry, gtk_get_current_event_time());
 	return;
@@ -44,7 +46,11 @@ entry_added (IndicatorObject * io, IndicatorObjectEntry * entry, gpointer user_d
 	g_debug("Signal: Entry Added");
 
 	GtkWidget * menuitem = gtk_menu_item_new();
+#if GTK_CHECK_VERSION(3,0,0)
+	GtkWidget * hbox = gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 3);
+#else
 	GtkWidget * hbox = gtk_hbox_new(FALSE, 3);
+#endif
 
 	if (entry->image != NULL) {
 		gtk_box_pack_start(GTK_BOX(hbox), GTK_WIDGET(entry->image), FALSE, FALSE, 0);
@@ -95,7 +101,11 @@ entry_removed (IndicatorObject * io, IndicatorObjectEntry * entry, gpointer user
 static void
 menu_show (IndicatorObject * io, IndicatorObjectEntry * entry, guint timestamp, gpointer user_data)
 {
-	g_debug("Show Menu: %s", entry->label != NULL ? gtk_label_get_text(entry->label) : "No Label");
+	if (entry != NULL) {
+		g_debug("Show Menu: %s", entry->label != NULL ? gtk_label_get_text(entry->label) : "No Label");
+	} else {
+		g_debug("Show Menu: (null)");
+	}
 	return;
 }
 
@@ -143,6 +153,9 @@ destroy (gpointer data)
 int
 main (int argc, char ** argv)
 {
+	/* Make sure we don't proxy to ourselves */
+	g_unsetenv("UBUNTU_MENUPROXY");
+
 	gtk_init(&argc, &argv);
 
 	if (argc != 2) {
